@@ -31,16 +31,14 @@ import os
 import re
 import tokenize
 from distutils import sysconfig
+import sys
 
 import pyflakes.api
 import pyflakes.messages
 import pyflakes.reporter
 
 
-__version__ = '0.4.1'
-
-
-PYFLAKES_BIN = 'pyflakes'
+__version__ = '0.6'
 
 
 ATOMS = frozenset([tokenize.NAME, tokenize.NUMBER, tokenize.STRING])
@@ -105,6 +103,14 @@ def unused_variable_line_numbers(messages):
 
 def check(source):
     """Return messages from pyflakes."""
+    if sys.version_info[0] == 2 and isinstance(source, unicode):
+        # Convert back to original byte string encoding, otherwise pyflakes
+        # call to compile() will complain. See PEP 263.
+        from StringIO import StringIO
+        string_io = StringIO(source.encode('utf-8'))
+        encoding = _detect_encoding(string_io.readline)
+        source = source.encode(encoding)
+
     reporter = ListReporter()
     try:
         pyflakes.api.check(source, filename='<string>', reporter=reporter)
@@ -118,7 +124,7 @@ class StubFile(object):
     """Stub out file for pyflakes."""
 
     def write(self, *_):
-        """Stub out write()."""
+        """Stub out."""
 
 
 class ListReporter(pyflakes.reporter.Reporter):
@@ -136,7 +142,7 @@ class ListReporter(pyflakes.reporter.Reporter):
         self.messages = []
 
     def flake(self, message):
-        """Override Reporter.flake()."""
+        """Accumulate messages."""
         self.messages.append(message)
 
 
@@ -161,21 +167,20 @@ def extract_package_name(line):
 
 def multiline_import(line, previous_line=''):
     """Return True if import is spans multiples lines."""
-    for symbol in '\\();:':
+    for symbol in '()':
         if symbol in line:
             return True
 
-    # Check for doctests.
-    stripped_line = line.strip()
-    if stripped_line and not stripped_line[0].isalnum():
+    # Ignore doctests.
+    if line.lstrip().startswith('>'):
         return True
 
-    return previous_line.rstrip().endswith('\\')
+    return multiline_statement(line, previous_line)
 
 
 def multiline_statement(line, previous_line=''):
     """Return True if this is part of a multiline statement."""
-    for symbol in '\\:':
+    for symbol in '\\:;':
         if symbol in line:
             return True
 
@@ -183,7 +188,7 @@ def multiline_statement(line, previous_line=''):
     try:
         list(tokenize.generate_tokens(sio.readline))
         return previous_line.rstrip().endswith('\\')
-    except (tokenize.TokenError, SyntaxError):
+    except (SyntaxError, tokenize.TokenError):
         return True
 
 
@@ -193,6 +198,7 @@ def break_up_import(line):
     assert '(' not in line
     assert ')' not in line
     assert ';' not in line
+    assert '#' not in line
 
     newline = get_line_ending(line)
     if not newline:
@@ -201,17 +207,11 @@ def break_up_import(line):
     (indentation, imports) = re.split(pattern=r'\bimport\b',
                                       string=line, maxsplit=1)
 
-    if '#' in imports:
-        (imports, comment) = imports.split('#', 1)
-        comment = '  # ' + comment.strip()
-    else:
-        comment = ''
-
     indentation += 'import '
     assert newline
 
-    return ''.join([indentation + i.strip() + comment + newline
-                    for i in imports.split(',')])
+    return ''.join([indentation + i.strip() + newline
+                    for i in sorted(imports.split(','))])
 
 
 def filter_code(source, additional_imports=None,
@@ -237,7 +237,7 @@ def filter_code(source, additional_imports=None,
     sio = io.StringIO(source)
     previous_line = ''
     for line_number, line in enumerate(sio.readlines(), start=1):
-        if line.strip().lower().endswith('# noqa'):
+        if '#' in line:
             yield line
         elif line_number in marked_import_line_numbers:
             yield filter_unused_import(
@@ -272,7 +272,9 @@ def filter_unused_import(line, remove_all_unused_imports, imports,
 
 def filter_unused_variable(line, previous_line=''):
     """Return line if used, otherwise return None."""
-    if multiline_statement(line, previous_line):
+    if re.match(r'^\s*except [\s,()\w]+ as \w+:$', line):
+        return re.sub(r' as \w+:$', ':', line, count=1)
+    elif multiline_statement(line, previous_line):
         return line
     elif line.count('=') == 1:
         split_line = line.split('=')
@@ -293,9 +295,14 @@ def is_literal_or_name(value):
         ast.literal_eval(value)
         return True
     except (SyntaxError, ValueError):
-        # Support removal of variables on the right side. But make sure
-        # there are no dots, which could mean an access of a property.
-        return re.match(r'^\w+\s*$', value)
+        pass
+
+    if value.strip() in ['dict()', 'list()', 'set()']:
+        return True
+
+    # Support removal of variables on the right side. But make sure
+    # there are no dots, which could mean an access of a property.
+    return re.match(r'^\w+\s*$', value)
 
 
 def useless_pass_line_numbers(source):
@@ -337,7 +344,7 @@ def filter_useless_pass(source):
     """Yield code with useless "pass" lines removed."""
     try:
         marked_lines = frozenset(useless_pass_line_numbers(source))
-    except (tokenize.TokenError, SyntaxError):
+    except (SyntaxError, tokenize.TokenError):
         marked_lines = frozenset()
 
     sio = io.StringIO(source)
@@ -428,15 +435,24 @@ def detect_encoding(filename):
     """Return file encoding."""
     try:
         with open(filename, 'rb') as input_file:
-            from lib2to3.pgen2 import tokenize as lib2to3_tokenize
-            encoding = lib2to3_tokenize.detect_encoding(input_file.readline)[0]
+            encoding = _detect_encoding(input_file.readline)
 
             # Check for correctness of encoding.
             with open_with_encoding(filename, encoding) as input_file:
                 input_file.read()
 
         return encoding
-    except (SyntaxError, LookupError, UnicodeDecodeError):
+    except (LookupError, SyntaxError, UnicodeDecodeError):
+        return 'latin-1'
+
+
+def _detect_encoding(readline):
+    """Return file encoding."""
+    try:
+        from lib2to3.pgen2 import tokenize as lib2to3_tokenize
+        encoding = lib2to3_tokenize.detect_encoding(readline)[0]
+        return encoding
+    except (LookupError, SyntaxError, UnicodeDecodeError):
         return 'latin-1'
 
 
@@ -492,7 +508,7 @@ def main(argv, standard_out, standard_error):
     while filenames:
         name = filenames.pop(0)
         if args.recursive and os.path.isdir(name):
-            for root, directories, children in os.walk(name):
+            for root, directories, children in os.walk(unicode(name)):
                 filenames += [os.path.join(root, f) for f in children
                               if f.endswith('.py') and
                               not f.startswith('.')]
